"""Setting up light controllers and monitoring their state"""

import re
import data
from flask import (Blueprint, flash, g, redirect, render_template, request,
                   url_for)
from werkzeug.exceptions import abort

bp = Blueprint('controllers', __name__)


@bp.route('/', methods=('GET', ))
def list():
    lst = get_list()
    return render_template('controllers/index.html',
                           config=lst,
                           footer=data.get_info())


@bp.route('/override', methods=('POST', ))
def update():
    port = None
    enabled = 'True'
    registers = []

    for k, v in request.form.items():
        if k == 'port':
            port = int(v)
            continue
        if k == 'enabled':
            enabled = v
            continue
        registers.append({'Name': k, 'Value': v})

    controllers = data.get_controllers()
    existing = _find(port, controllers)
    if existing is None:
        abort(404)

    current_regs = data.get_controller_registers(existing['Port'])
    for r in registers:
        reg = next(filter(lambda x: x['Name'] == r['Name'], current_regs), r)
        display_as = reg.get('DisplayAs', 'DropdownVariants')
        if display_as == 'RawBinary' and _is_binary_str(r['Value']):
            r['Value'] = int(r['Value'], 2)
        else:
            r['Value'] = int(r['Value'])

    data.set_controller_registers(existing['Type'], existing['Port'],
                                  registers)
    enabled = enabled == 'True'

    if existing['Enabled'] != enabled:
        existing['Enabled'] = enabled
        data.set_controllers(controllers)

    return redirect(url_for("controllers.list"))


def _find(port: int, controllers: list) -> dict:
    controller = next(filter(lambda c: c['Port'] == port, controllers), None)
    return controller


@bp.route('/<int:port>/delete', methods=('POST', ))
def delete(port):
    controllers = data.get_controllers()
    existing = _find(port, controllers)
    if existing is None:
        abort(404)
    data.remove_controller(port)
    return redirect(url_for("controllers.list"))


@bp.route('/add', methods=('POST', 'GET'))
def add():
    types = []
    controllers = data.get_controllers()
    models = data.get_controller_types()
    for t in models:
        types.append(t['Name'])

    if request.method == 'POST':
        t = request.form['type']
        p = int(request.form["port"])
        e = request.form['enabled'] == 'True'
        controller = _find(p, controllers)
        if p < 1024 or p > 65536:
            flash('Port should be in range 1025-65536')
        elif controller:
            flash('This port is already assigned to another controller!')
        elif t not in types:
            flash(f'Type {t} was not found')
        else:
            controllers.append({
                'Type': t,
                'Enabled': e,
                'Ip': '127.0.0.1',
                'Port': p
            })
            data.set_controllers(controllers)
            return redirect(url_for('controllers.list'))

    return render_template('controllers/create.html',
                           types=types,
                           footer=data.get_info())


def _get_current_value(reg: dict) -> str:
    if 'DisplayAs' in reg and reg['DisplayAs'] == 'RawBinary':
        return f'{reg["CurrentValue"]:b}'.zfill(16)
    else:
        return reg['CurrentValue']


def _get_value_name(reg: dict) -> str:
    value = reg['CurrentValue']
    display_as = reg.get('DisplayAs', 'DropdownVariants')
    if display_as == 'RawBinary':
        return value
    elif display_as == 'RawInteger':
        return ''
    else:
        for possible in reg['PossibleValues']:
            for k, v in possible.items():
                if v == value:
                    return k
        return 'Undefined'


def get_list() -> dict:
    controllers = data.get_controllers()

    for controller in controllers:
        controller['Enabled'] = _to_status(controller['Enabled'])
        controller['Registers'] = []
        registers = data.get_controller_registers(controller['Port'])

        for reg in registers:
            controller['Registers'].append({
                'Name':
                reg['Name'],
                'Value':
                _get_current_value(reg),
                'ValueStr':
                _get_value_name(reg),
                'Address':
                reg['Address'],
                'Type':
                reg['Type'],
                'PossibleValues':
                reg['PossibleValues'],
                'CanOverride':
                reg['CanOverride'],
                'DisplayAs':
                reg.get('DisplayAs', 'DropdownVariants')
            })

    return controllers


def _to_status(enabled: bool) -> str:
    if enabled:
        return 'Yes'
    else:
        return 'No'


def _is_binary_str(s: str) -> bool:
    pattern = r'^[01]+$'
    return re.match(pattern, s) is not None
